

#include <avr/io.h>
#include <util/delay.h>


void delay_ms(unsigned int ms)
/* delay for a minimum of <ms> */
{
  // we use a calibrated macro. This is more
  // accurate and not so much compiler dependent
  // as self made code.
  while(ms){
    _delay_ms(0.98);
    ms--;
  }
}



#if defined (__AVR_ATmega168__) || defined (__AVR_ATmega168A__)
// ATMEGA168
#define SPIPORT PORTB
#define SPIDDR  DDRB
#define SPIMISO 4       // Port B bit 5 (pin7): data in (data from MMC)
#define SPIMOSI 3       // Port B bit 6 (pin6): data out (data to MMC)
#define SPICLK  5       // Port B bit 7 (pin8): clock
#define SPICS   2       // Port B bit 4 (pin5: chip select for MMC
#endif

#if defined (__AVR_ATmega8U2__) || defined (__AVR_ATmega32U2__) || defined (__AVR_ATmega16U2__) ||defined (__AVR_AT90USB82__) || defined (__AVR_AT90USB162__) 
#define SPIPORT PORTB
#define SPIDDR  DDRB
#define SPIMISO 3       // PB3
#define SPIMOSI 2       // PB2
#define SPICLK  1       // PB1
#define SPICS   0       // PB0
#endif


unsigned char init; 

void SPIinit(void)
{
  SPIDDR &= ~_BV(SPIMISO);    // set port B SPI data input to input
  SPIDDR |=  _BV(SPICLK);    // set port B SPI clock to output
  SPIDDR |=  _BV(SPIMOSI);  // set port B SPI data out to output 
  SPIDDR |=  _BV(SPICS);   // set port B SPI chip select to output
  SPCR    =  _BV(SPE) | _BV(MSTR) | _BV(SPR1) | _BV(SPR0);
  SPIPORT |= _BV (SPICS); // set chip select to High (slave not active)
  init = 1; 
}

char SPI(char d) 
{  // send character over SPI
  char received = 0;
  SPDR = d;
  while(!(SPSR & _BV(SPIF)));
  received = SPDR;
  return (received);
}

void SPI_startpkt (void)
{
  SPIPORT &= ~_BV (SPICS); // set chip select to low(slave active)
}

void SPI_endpkt (void)
{
  SPIPORT |= _BV (SPICS); // set chip select to low(slave active)
}



static unsigned char addr = 0x86;

#define WAIT 5

void set_servo (unsigned char snum, unsigned char sval)
{
  SPI_startpkt ();
  _delay_us (5);
  SPI (addr);
  _delay_us (5);
  SPI (0x20+snum);
  _delay_us (5);
  SPI (sval);
  _delay_us (5);
  SPI_endpkt ();
}


int main (void)
{
  int t;

  SPIinit (); 
  t = 0;
  while (1) {
    if ( (t & 0x1ff) < 0x100) {
      set_servo (0, t);
      set_servo (1, 0x100-t);
    } else {
      set_servo (0, 0x200-t);
      set_servo (1, t-0x100);
    }

    if ( (t & 0x3ff) < 0x200) {
      set_servo (2, t/2);
      set_servo (3, 0x100-(t/2));
    } else {
      set_servo (2, 0x200-(t/2));
      set_servo (3, (t/2)-0x100);
    }

    if (t >= 0x3ff) t = 0;
    else            t++; 
    delay_ms (20);
  }
}
